#!/bin/bash
# Copyleft (C) 2010 by <tv>
#
# ./makeself.sh ./install.sh "Script d'installation pour Mandriva Linux (bts iris) by tv" ./setup.sh
#

### Informations sur l'installation
NAME="qhelloworld"
VERSION="1.0.0"
SUMMARY="Affiche \"Hello world !\" dans une boite de dialogue"
LICENCE="GPL"
AUTEUR="Thierry Vaira <thierry.vaira@orange.fr>"

HAVE_MENU_KDE="yes"
ICON="qdevelop"
CATEGORY="Qt;Development;IDE;"
MIMETYPE="application/x-qdevelop;"
####################################

### liste des ressources  installer
BIN_FILES="${NAME}"
DOC_FILES="README NEWS COPYING AUTHORS TODO ChangeLog"
MAN_FILES="man1/*"
MAN_FILES="man1/qhelloworld.1.*"
EXTRA_FILES=""
####################################

### repertoires
BINDIR="usr/bin"
DATADIR="usr/share"
DOCDIR="${DATADIR}/doc/${NAME}"
MANDIR="${DATADIR}/man"
EXTRADIR="${DATADIR}/${NAME}"
TMPDIR="/tmp"
####################################

### outils
CAT="cat"
SED="sed"
INSTALL_FILE="install -m 644"
INSTALL_PROGRAM="install -m 755"
DEL_FILE="rm -f"
DEL_DIR="rmdir"
CHK_DIR_EXISTS="test -d"
CHK_FILE_EXISTS="test -f"
MKDIR="mkdir -p"
DIRMODE="755"
MKTEMP="mktemp"
MKTEMP_FILE="mktemp -q"
###################################

function interrupt_quit() 
{
   $DEL_FILE $TMP_FILE > /dev/null 2>&1
	#echo -e "\nInstallation annulee !"
	#exit 1
}

function init()
{
   # temp file
   # mktemp install ?
   if [ -x "`which ${MKTEMP} 2>&-`" ]
   then
      TMP_FILE="$(${MKTEMP_FILE} ${TMPDIR}/$0.XXXXXX)" || exit 1
   else
      TMP_FILE="${TMPDIR}/$(basename $0).$$"
      ! $CHK_FILE_EXISTS $TMP_FILE && $TOUCH $TMP_FILE > /dev/null  2>&1 || exit 1
   fi

   distribution=`${CAT} /etc/release 2> /dev/null | tr "[:upper:]" "[:lower:]" 2> /dev/null`

   # by Bjarni R. Einarsson <bre@netverjar.is>
   GUESS_XTERMS="xterm dtterm rxvt kvt eterm konsole"
   #PREFER_X=yes
   PREFER_TTY=yes
   for a in $GUESS_XTERMS; do
      if which $a 2>/dev/null >/dev/null; then
         XTERM=$a
      fi
   done
   HAVE_TTY=
   INTERFACE=null
   if stty >/dev/null 2>/dev/null; then
      HAVE_TTY=1
      INTERFACE=tty
   fi
   if [ "$DISPLAY" != "" ]; then
      if [ "$HAVE_TTY" = "" -a "$XTERM_LOOP" != "$PREFER_TTY" ]; then
         export XTERM_LOOP="$PREFER_TTY"
         exec $XTERM -title $(basename $0) -e "$0" ${@+"$@"}
      fi
      [ "$PREFER_X" != "" -o "$HAVE_TTY" = "" ] && INTERFACE=X
   fi
   XTERM_LOOP=
   export HAVE_TTY XTERM_LOOP
   case $INTERFACE in   
      X)
         if [ "$DISPLAY" = "" ]; then
            echo "Interface graphique non disponible (variable DISPLAY vide) !"
            exit 1
         fi
         if !which xmessage 2>/dev/null >/dev/null; then
            echo "Le programme xmessage n'est pas install !"
            exit 1
         fi
         CAT="xmessage -buttons okay:0 -title setup.sh -center -default okay -file -"
         CLEARSCR=true
         ;;
      tty)
         CAT=cat
         CLEARSCR=clear
         ;;
      null)
         CAT=cat
         ;;
   esac
}

function query()
{
   QUESTION=$1
   DEFAULT=$2  

   case $INTERFACE in
      tty)
         echo -n "$QUESTION"
         if [ 0 -eq $DEFAULT ]; then
            echo -n " [O/n/q] "
         else
            echo -n " [o/N/q] "
         fi
         read -s -n 1 R <&1
         echo

         [ "$R" = "O" -o "$R" = "o" ] && return 0
         [ "$R" = "N" -o "$R" = "n" ] && return 1
         [ "$R" = "Q" -o "$R" = "q" ] && echo -e "\x1B[0m" && exit 1
         return $DEFAULT
         ;;
      X) 
         if [ 0 -eq $DEFAULT ]; then
            DL=oui
         else
            DL=non
         fi
         xmessage \
            -title $(basename $0) -center \
            -buttons "oui:0,non:1,quitter:2" \
            -default $DL "$QUESTION"

         R=$?
         [ $R -eq 2 ] && exit 0
         return $R
         ;;
      *)
         if [ 0 -eq $DEFAULT ]; then
            DEF=oui
         else
            DEF=non
         fi
         echo "$QUESTION ($DEF !)" | cat 
         return $DEFAULT
         ;;
   esac
}

function root_check()
{
   # seul root peut executer ce script :)
   #id | grep "uid=0(root)" > /dev/null  2>&1
   #if [ $? != "0" ]
   #then
   #   echo -e "\x1B[49;31;1mERREUR: ce script ne peut etre execute que sous le compte root !\x1B[0m"
   #   echo "";
   #   exit 1;
   #fi

   #$CLEARSCR

   echo -e "\x1B[49;31;1m"
   WARNING="\
WARNING:
    Installing programs as root can seriously damage your system!
    Unless you obtained it from a trusted source, you run the risk 
    of installing trojan horses or virii - not to mention the fact
    that the installation sequence itself might inadvertantly harm
    your system.

    Installing as a normal user and examining the program (perhaps 
    later re-installing it as root) is recommended for most 
    applications."
   
   id | grep "uid=0(root)" > /dev/null  2>&1
   if [ $? != "0" ]; then
   #if [ "$UID" != "0" ]; then

      QUESTION="
You are not running $(basename $0) as root (the system administrator),
which means you cannot perform a system-wide installation of this
program.

At this point you may (if you know the root password) complete the
installation as root, or just continue with your normal privileges 
and install the program in your home directory.

$WARNING

Continue as root ?"

      if query "$QUESTION" 1; then
         if [ "$HAVE_TTY" = "" -a "$DISPLAY" != "" ]; then
            : exec $XTERM -title $(basename $0) -e su -c "$0 -i $INTERFACE --asroot"
         else
            exec su -c "$0 -i $INTERFACE --asroot"
         fi
      fi
      echo -e "\x1B[0m"
      echo "";
      exit 1;
      #ou on continue ? TODO
      export INSTALL_IN_HOME=yes 
   else
      QUESTION="
$WARNING

Continue as root ?"
      
      if ! query "$QUESTION" 1; then
         echo -e "\x1B[0m"
         exit 0
      fi
   fi
   echo -e "\x1B[0m"
   return 0
}

function say_hello()
{
   $CLEARSCR

   echo -e "\x1B[49;34;1m
=====================================================
Copyleft (C) 2010 <tv>

Script d'installation pour Mandriva Linux (bts iris)
=====================================================\x1B[0m\n"

   echo -n "Vous utilisez une distribution "
   echo -e "\x1B[49;34;1m$distribution\x1B[0m\n"

   MESSAGE="Vous etes sur le point d'installer

Nom: $NAME
Version: $VERSION
Description: $SUMMARY
Auteur: $AUTEUR
Licence: $LICENCE

Continuer ?"

   if ! query "$MESSAGE" 0; then
      exit 1
   fi
}

function verif_size()
{
   DESTDIR="$1"
   shift
   BASEDIR="$1"
   shift
   basedir="${BASEDIR-pwd}"
   totalSizeKB=0
   for file in ${*}
   do
        fileSizeKB=0
        name=$(echo $basedir/$file | $SED -e 's/[^[:alnum:]\.\_\-\/]//g')
        $CHK_FILE_EXISTS $name && fileSizeKB=`ls -l $name | awk '{print $(NF-3)}' | tail -1`
        totalSizeKB=`expr $totalSizeKB + $fileSizeKB`
   done
   totalSizeKB=`expr $totalSizeKB / 1024`

   echo "Espace total requis : $totalSizeKB kB"
   echo -n "Espace disponible   : "
   freespace=`df -k $DESTDIR | tail -1 | awk '{print $(NF-2)}'`
   echo "$freespace kB"
   if [ $totalSizeKB -gt $freespace ] ; then
      return 1
   fi
   return 0
}

function install_dir() 
{
   # from mkinstalldirs by Noah Friedman <friedman@prep.ai.mit.edu>
   dirmode=$1
   shift
   for f
   do
      if test -d "$f"; then
         shift
      else
         break
      fi
   done

   case $# in
   0) return 0 ;;
   esac

   case $dirmode in
  '')
    if $MKDIR --version . >/dev/null 2>&1 && test ! -d ./--version; then
      #echo "${MKDIR} -- $*"
      $MKDIR -- "$@"
    else
      # On NextStep and OpenStep, the `mkdir' command does not
      # recognize any option.  It will interpret all options as
      # directories to create, and then abort because `.' already
      # exists.
      test -d ./-p && $DEL_DIR ./-p
      test -d ./--version && $DEL_DIR ./--version
    fi
    ;;
  *)
    if mkdir -m "$dirmode" -p --version . >/dev/null 2>&1 &&
       test ! -d ./--version; then
      #echo "mkdir -m $dirmode -p -- $*"
      mkdir -m "$dirmode" -p -- "$@"
    else
      # Clean up after NextStep and OpenStep mkdir.
      for d in ./-m ./-p ./--version "./$dirmode";
      do
        test -d $d && $DEL_DIR $d
      done
    fi
    ;;
   esac
	echo ""
   return 0
}

function install_file() 
{
	echo "Etape $1 : Installation des fichiers"
   # installation : bin
   if [ x"$BIN_FILES" != x ]; then
      verif_size "/${BINDIR}" "${BINDIR}" $BIN_FILES
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[49;31;1mErreur : manque d'espace disque !\x1B[0m"
         return 1
      fi
      echo ""
      install_dir $DIRMODE "/${BINDIR}"
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[60G\x1B[49;31;1mIErreur : impossible de crer le rpertoire "/${BINDIR}"\x1B[0m" 
         return 1
      fi
      for file in $BIN_FILES
	   do
         echo -n "Installation du fichier $file dans /${BINDIR}"
         INSTALL_CMD="${INSTALL_PROGRAM} "${BINDIR}/$file" "/${BINDIR}/$file""
         $CHK_DIR_EXISTS "/${BINDIR}" && $INSTALL_CMD > /dev/null 2>&1
         $CHK_FILE_EXISTS "/${BINDIR}/$file" || echo -e "\x1B[60G\x1B[49;31;1m[ERREUR]\x1B[0m"
         $CHK_FILE_EXISTS "/${BINDIR}/$file" && echo -e "\x1B[60G\t\t\x1B[49;32;1m[OK]\x1B[0m"
         $CHK_FILE_EXISTS "/${BINDIR}/$file" && echo "$DEL_FILE /${BINDIR}/$file" >> $TMP_FILE
      done
      echo ""
   fi

   # installation : doc
   if [ x"$DOC_FILES" != x ]; then
      install_dir $DIRMODE "/${DOCDIR}"
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[60G\x1B[49;31;1mIErreur : impossible de crer le rpertoire "/${DOCDIR}"\x1B[0m" 
         return 1
      fi
      verif_size "/${DOCDIR}" "." $DOC_FILES
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[49;31;1mErreur : manque d'espace disque !\x1B[0m"
         return 1
      fi
      echo ""
      for file in $DOC_FILES
	   do
         echo -n "Installation du fichier $file dans /${DOCDIR}"
         INSTALL_CMD="${INSTALL_FILE} $file "/${DOCDIR}/${file}""
         $CHK_DIR_EXISTS "/${DOCDIR}" && $INSTALL_CMD > /dev/null 2>&1
         $CHK_FILE_EXISTS "/${DOCDIR}/$file" || echo -e "\x1B[60G\x1B[49;31;1m[ERREUR]\x1B[0m"
         $CHK_FILE_EXISTS "/${DOCDIR}/$file" && echo -e "\x1B[60G\t\t\x1B[49;32;1m[OK]\x1B[0m"
         $CHK_FILE_EXISTS "/${DOCDIR}/$file" && echo "$DEL_FILE /${DOCDIR}/$file" >> $TMP_FILE
      done
      $CHK_DIR_EXISTS "/${DOCDIR}" && echo "$DEL_DIR /${DOCDIR}" >> $TMP_FILE
      echo ""
   fi
   # installation : man
   if [ x"$MAN_FILES" != x ]; then
      verif_size "/${MANDIR}" "${MANDIR}" $MAN_FILES
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[49;31;1mErreur : manque d'espace disque !\x1B[0m"
         return 1
      fi
      echo ""
      install_dir $DIRMODE "/${MANDIR}"
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[60G\x1B[49;31;1mIErreur : impossible de crer le rpertoire "/${MANDIR}"\x1B[0m" 
         return 1
      fi
      for file in "${MANDIR}/${MAN_FILES}"
	   do
         echo -n "Installation des pages man dans /${MANDIR}"
         for f in $file 
         do
            dstdir=`echo $f | $SED -e 's,[^/]*$,,;s,/$,,;s,^$,.,'`
         done
         INSTALL_CMD="${INSTALL_FILE} ${file} "/${dstdir}""
         $CHK_DIR_EXISTS "/${dstdir}" && $INSTALL_CMD > /dev/null 2>&1
         if [ $? -eq 1 ] ; then
            echo -e "\x1B[60G\x1B[49;31;1m[ERREUR]\x1B[0m"
         else
            echo -e "\x1B[60G\t\t\x1B[49;32;1m[OK]\x1B[0m"
            echo "$DEL_FILE /${file}" >> $TMP_FILE
         fi
      done
      echo ""
   fi
   # installation : extra
   if [ x"$EXTRA_FILES" != x ]; then
      verif_size "/${EXTRADIR}" "${EXTRADIR}" $EXTRA_FILES
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[49;31;1mErreur : manque d'espace disque !\x1B[0m"
         return 1
      fi
      echo ""
      install_dir $DIRMODE "/${EXTRADIR}"
      if [ $? -eq 1 ] ; then
         echo -e "\x1B[60G\x1B[49;31;1mIErreur : impossible de crer le rpertoire "/${EXTRADIR}"\x1B[0m" 
         return 1
      fi
      for file in $EXTRA_FILES
	   do
         echo -n "Installation du fichier $file dans /${EXTRADIR}"
         INSTALL_CMD="${INSTALL_FILE} $file "/${EXTRADIR}/${file}""
         $CHK_DIR_EXISTS "/${EXTRADIR}" && $INSTALL_CMD > /dev/null 2>&1
         $CHK_FILE_EXISTS "/${EXTRADIR}/$file" || echo -e "\x1B[60G\x1B[49;31;1m[ERREUR]\x1B[0m"
         $CHK_FILE_EXISTS "/${EXTRADIR}/$file" && echo -e "\x1B[60G\t\t\x1B[49;32;1m[OK]\x1B[0m"
         $CHK_FILE_EXISTS "/${EXTRADIR}/$file" && echo "$DEL_FILE /${EXTRADIR}/$file" >> $TMP_FILE
      done
      echo ""
   fi

   return 0
}

function install_menu()
{
   if [ x"$HAVE_MENU_KDE" != x ]; then
	   echo -n "Etape $1 : Installation du menu dans KDE"
	   $CHK_DIR_EXISTS "./${DATADIR}/applications" || $MKDIR "./${DATADIR}/applications" > /dev/null 2>&1
      cat > "./${DATADIR}/applications/mandriva-${NAME}.desktop" <<EOF
[Desktop Entry]
Encoding=UTF-8
Name=${NAME}
GenericName=Application
Comment=${SUMMARY}
Exec=/${BINDIR}/${NAME}
Icon=${ICON}
Terminal=false
Type=Application
StartupNotify=true
MimeType=${MIMETYPE}
Categories=${CATEGORY}
EOF

   $CHK_FILE_EXISTS "./${DATADIR}/applications/mandriva-${NAME}.desktop" && desktop-file-install \
--add-category="${CATEGORY}" \
--add-mime-type="${MIMETYPE}" \
--add-only-show-in="KDE" \
--dir /${DATADIR}/applications ./${DATADIR}/applications/* > /dev/null 2>&1

      if [ $? -eq 1 ] ; then
         echo -e "\x1B[60G\x1B[49;31;1m[ERREUR]\x1B[0m"
      else
         echo -e "\x1B[60G\t\t\x1B[49;32;1m[OK]\x1B[0m"
         echo "$DEL_FILE /${DATADIR}/applications/mandriva-${NAME}.desktop" >> $TMP_FILE
      fi
   fi
   return 0
}

function create_uninstaller()
{
   UNINST_PATH="/usr/local/sbin"
   UNINST_SH="uninstall-$NAME-$VERSION.sh"
   install_dir $DIRMODE "/${UNINST_PATH}"
   UNINST="$UNINST_PATH/$UNINST_SH"

   cat <<-tac > "$UNINST"
#!/bin/sh
#
if ! stty >/dev/null 2>/dev/null; then
   if [ x"\$DISPLAY" != x -a x"\$XTERM_LOOP" = x ]; then
      export XTERM_LOOP=1
      exec $XTERM -title "\$0" -e "\$0"
   fi
fi
      
echo "Ce script desinstalle $NAME $VERSION"
echo
echo "Appuyez sur une touche pour continuer ou CTRL-C pour quitter"
read -s -n 1 TOUCHE <&1
tac
      $CAT $TMP_FILE >> "$UNINST"
      echo "$DEL_FILE $UNINST" >> "$UNINST"
      chmod +x "$UNINST"
      echo
      echo "Un desinstalleur a ete cree dans $UNINST"
}

################################################################################
#	Main
################################################################################

# Initialisation
trap interrupt_quit 1 2 3 9 15 EXIT ERR

init

say_hello

root_check

# Demarrage de l'installation
(( etape = 1 ))
# Etape : installe les fichiers
install_file $etape && (( etape += 1 ))

# Etape : creation du menu dans KDE
install_menu $etape && (( etape += 1 ))

# Fin de l'installation
create_uninstaller

echo " "
echo "Installation terminee !"
exit 0
